@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2015  Ruby-GNOME2 Project Team
+# Copyright (C) 2012-2017  Ruby-GNOME2 Project Team
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -118,9 +118,11 @@ module GObjectIntrospection
         klass = self.class.define_struct(info.size, name, @base_module,
                                          :parent => options[:parent])
       else
+        size = info.size
+        size = nil if size.zero?
         klass = self.class.define_class(info.gtype, name, @base_module,
                                         :parent => options[:parent],
-                                        :size   => info.size)
+                                        :size   => size)
       end
       load_fields(info, klass)
       load_methods(info, klass)
@@ -133,7 +135,7 @@ module GObjectIntrospection
         method_infos = info.methods.find_all do |method_info|
           method_info.is_a?(MethodInfo)
         end
-        unless methods.empty?
+        unless method_infos.empty?
           base_class = @base_module.const_get(base_class_name)
           load_method_infos(method_infos, base_class.singleton_class)
         end
@@ -233,29 +235,48 @@ module GObjectIntrospection
       field_info.name
     end
 
-    def load_field(info, i, field_info, klass)
+    def load_field(info, i, field_info, klass, options={})
       name = field_name(field_info, klass)
       flags = field_info.flags
 
-      if flags.readable?
-        if field_info.type.tag == TypeTag::BOOLEAN
-          reader_method_name = "#{name}?"
-        else
-          reader_method_name = name
-        end
-        remove_existing_method(klass, reader_method_name)
-        klass.__send__(:define_method, reader_method_name) do ||
-          info.get_field_value(self, i)
-        end
+      readable = options[:readable]
+      readable = flags.readable? if readable.nil?
+      if readable
+        reader_method_name = rubyish_field_reader_name(field_info, name)
+        load_field_reader(info, i, field_info, klass, name, reader_method_name)
       end
 
-      if flags.writable?
-        klass.__send__(:define_method, "#{name}=") do |value|
-          info.set_field_value(self, i, value)
+      writable = options[:writable]
+      writable = flags.writable? if writable.nil?
+      if writable
+        load_field_writer(info, i, field_info, klass, name, "#{name}=")
+      end
+    end
+
+    def load_field_reader(info, i, field_info, klass, name, method_name)
+      remove_existing_method(klass, method_name)
+
+      need_number_to_bool_convert = false
+      if name.start_with?("is_") and
+          field_info.type.tag == TypeTag::UINT32
+        need_number_to_bool_convert = true
+      end
+      klass.__send__(:define_method, method_name) do ||
+        value = info.get_field_value(self, i)
+        if need_number_to_bool_convert
+          value != 0
+        else
+          value
         end
       end
     end
 
+    def load_field_writer(info, i, field_info, klass, name, method_name)
+      klass.__send__(:define_method, method_name) do |value|
+        info.set_field_value(self, i, value)
+      end
+    end
+
     def load_methods(info, klass)
       grouped_methods = info.methods.group_by do |method_info|
         method_info.class
@@ -413,8 +434,17 @@ module GObjectIntrospection
       if options[:prefix]
         name = name.gsub(/\A#{Regexp.escape(options[:prefix])}/, "")
       end
+      if name == "initialize"
+        name += "_raw"
+      end
       return_type = function_info.return_type
       return_type_tag = return_type.tag
+      if return_type_tag == TypeTag::VOID
+        out_arg_tags = function_info.out_args.collect {|arg| arg.type.tag}
+        if out_arg_tags == [TypeTag::ARRAY, TypeTag::INT32]
+          return_type_tag = TypeTag::ARRAY
+        end
+      end
       case return_type_tag
       when TypeTag::BOOLEAN
         case name
@@ -443,7 +473,7 @@ module GObjectIntrospection
         else
           name
         end
-      when TypeTag::GLIST, TypeTag::GSLIST
+      when TypeTag::GLIST, TypeTag::GSLIST, TypeTag::ARRAY
         case name
         when /\A(?:list|get)_/
           if function_info.n_in_args.zero?
@@ -472,6 +502,21 @@ module GObjectIntrospection
       end
     end
 
+    def rubyish_field_reader_name(field_info, name)
+      case field_info.type.tag
+      when TypeTag::BOOLEAN
+        name.gsub(/\Ais_/, "") + "?"
+      when TypeTag::UINT32
+        if /\Ais_/ =~ name
+          "#{$POSTMATCH}?"
+        else
+          name
+        end
+      else
+        name
+      end
+    end
+
     def rubyish_class_name(info)
       name = info.name
       case info
